package bank3tier.lg; //LgSitzung.java

import java.sql.Date;
import java.util.List;

import bank3tier.db.DbApplication;
import bank3tier.db.DbJungeKundenQuery;
import bank3tier.db.DbObject;
import bank3tier.db.DbReicheKundenQuery;
import bank3tier.db.Pool;

/**Steht für eine Benutzersitzung. Enthält einen Konstruktor zur 
  Initialisierung der Sitzung sowie Operationen zum Erfassen 
  einiger Startobjekte, von denen ausgehend weitere Geschäftsobjekte mit Hilfe von
  Objektoperationen erfasst werden.
 * @author  Christoph Knabe
 */
public class LgSitzung {

	private final DbApplication _dbApplication;
    private final Pool _pool;
    
    /** Initializes the application.*/
    public LgSitzung() throws multex.Failure {
        try {
            this._dbApplication = new DbApplication(new Class[]{DbObject.class, LgKunde.class, LgKonto.class, LgKontozugriff.class});
            this._pool = this._dbApplication.getPool();
        } catch(final Exception ex){
            throw new multex.Failure("Failure initializing the business logic session", ex); 
        }
    }
    
    /**Returns the persistence Pool, this application is operating on*/
    /*package*/ Pool getPool(){return _pool;}
    
    /**Beginnt eine Applikationstransaktion.*/
    public void beginTransaction(){
        this._dbApplication.dbBeginTransaction();
    }
    
    /**Beendet eine Applikationstransaktion. Bestätigt Änderungen, wenn i_ok, verwirft andernfalls.*/
    public void endTransaction(final boolean i_ok){
        this._dbApplication.dbEndTransaction(i_ok);
    }
    
    /**Erzeugt ein neues Kundenobjekt als externe, geprüfte Operation.*/    
    public LgKunde kundeErfassen(final String i_name, final LgDatum i_geburt){
        try {
            final LgKunde kunde = new LgKunde();
            kunde.setzen(i_name, i_geburt);
            this._pool.insert(kunde);
            return kunde;
        }catch(final Exception ex){throw new multex.Failure(
            "Es war nicht möglich, den Kunden {0} zu erfassen", ex, i_name);
        }
    }

    /**Liefert alle Kunden*/
    public List<LgKunde> alleKundenSuchen(){
        try {
            final List<LgKunde> result = this._pool.findAll(LgKunde.class);
            return result;
        }catch(final Exception ex){throw new multex.Failure(
            "Failure searching all LgKunde-objects", ex);
        }
    }

    /**Liefert alle Kunden, die ab dem übergebenen Datum geboren wurden.
     * Suche in einer Tabelle, Abfrage nur eigener Attribute.*/
    public List<LgKunde> jungeKundenSuchen(final LgDatum i_datum){
        final Date date = i_datum.toDate();
        try {
            final List<LgKunde> result = _pool.findManyByQuery(LgKunde.class, DbJungeKundenQuery.class, new java.sql.Date[]{date});
            return result;
        }catch(final Exception ex){throw new multex.Failure(
            "Fehler beim Suchen von Kunden, die ab {0,date} geboren wurden", ex, date);
        }
    }

    /**Liefert den Kunden mit dem angegebenen Geburtsdatum
     * @throws RuntimeException Kein oder mehrere Kunden gefunden oder anderer Fehler
     */
    public LgKunde kundenSuchen(final LgDatum i_geburtsDatum)
    {
        final LgKunde result = this._pool.findOneBykey(LgKunde.class, LgKunde.geburtFIELD, i_geburtsDatum.toDate());
        return result;
    }
    
    /**Liefert alle LgKontozugriff-Objekte für Verbindungen Kunde als Inhaber mit seinem Konto
     * und druckt sie dabei aus.
    */
    public List<LgKontozugriff> printGetInhaberKontozugriffe()
    {
        final List<LgKontozugriff> result = this._pool.findManyByKey(LgKontozugriff.class, LgKontozugriff.inhaberFIELD, Boolean.TRUE);
        for(final LgKontozugriff kz: result){
            System.out.println(kz);
        }
        return result;
    }
    
    /**TODO Implement: Liefert alle Kontozugriff-Objekte die einen Kunden als Inhaber eines Kontos ausweisen, 
     * dessen Saldo größer oder gleich i_saldoCt ist.
     * */
    public List<LgKontozugriff> reicheKundenSuchen(final int i_saldoCt){
        try {
            final List<LgKontozugriff> result = this._pool.findManyByQuery(
                    LgKontozugriff.class, DbReicheKundenQuery.class, new Integer[]{new Integer(i_saldoCt)}
            );
            return result;
        }catch(final Exception ex){throw new multex.Failure(
            "Fehler beim Suchen von Kunden, die ab {0} Cent auf einem eigenen Konto haben", ex, new Integer(i_saldoCt)
        );}
    }

    /**Es wurde kein Kunde mit Geburtsdatum {0,date} gefunden*/
    public static class KeinKundeExc extends multex.Exc {
        private static final long serialVersionUID = 202789217592152949L;
        public KeinKundeExc(final LgDatum i_geburtDate){
            super("Es wurde kein Kunde mit Geburtsdatum {0,date} gefunden", i_geburtDate.toDate());
        }
    }
    

}